# -*- coding: utf-8 -*-
# @Author: wqshen
# @Email: wqshen91@gmail.com
# @Date: 2023/2/9 22:07
# @Last Modified by: wqshen

from typing import cast
from dataclasses import dataclass, fields


@dataclass
class AwxGridHead:
    satellite_name: str
    grid_element: int
    grid_byte: int
    grid_base: int
    grid_scale: int
    time_scale: int
    start_year: int
    start_month: int
    start_day: int
    start_hour: int
    start_minute: int
    end_year: int
    end_month: int
    end_day: int
    end_hour: int
    end_minute: int
    ul_lat: int
    ul_lon: int
    lr_lat: int
    lr_lon: int
    grid_unit: int
    reso_h: int
    reso_v: int
    size_h: int
    size_v: int
    has_land: int = 0
    land: int = 0
    has_cloud: int = 0
    cloud: int = 0
    has_water: int = 0
    water: int = 0
    has_ice: int = 0
    ice: int = 0
    has_quality: int = 0
    quality_up: int = 0
    quality_down: int = 0
    reserve: int = 0

    @staticmethod
    def dtype() -> str:
        return '=8s36h'

    def __post_init__(self):
        self._cast_fields_types()

    def _cast_fields_types(self):
        for field in fields(cast(dataclass, self)):
            field_value = getattr(self, field.name)
            if isinstance(field_value, bytes):
                field_value = field_value.rstrip(b'\x00').decode('utf8')
            setattr(self, field.name, field.type(field_value))

    def grid_elements_dict(self):
        return {0: 'numerical weather prediction',
                1: 'sea surface temperature (K) ',
                2: 'sea ice distribution (dimensionless) ',
                3: 'sea ice density (dimensionless)',
                4: 'outgoing longwave radiation (W/m2)',
                5: 'normalized vegetation index (dimensionless)',
                6: 'vegetation index ratio (dimensionless) ',
                7: 'snow distribution (dimensionless)',
                8: 'soil moisture (kg/m3)',
                9: 'sunshine (hour) ',
                10: 'cloud top height (hPa) ',
                11: 'cloud top temperature (K)',
                12: 'low cirrus (dimensionless)',
                13: 'high cirrus (dimensionless)',
                14: 'precipitation index (mm/1h) ',
                15: 'precipitation index (mm/6h)',
                16: 'precipitation index (mm/12h) ',
                17: 'precipitation index (mm/24h) ',
                18: 'upper tropospheric water vapor (relative humidity) (dimensionless)',
                19: 'brightness temperature',
                20: 'cloud amount (percentage)',
                21: 'cloud classification (dimensionless)',
                22: 'precipitation estimates (mm/6h)',
                23: 'precipitation estimates (mm/24h)',
                24: 'clear sky atmospheric precipitation (mm)',
                25: 'reserved',
                26: 'ground incident solar radiation (W/m2)',
                27: 'reserved ',
                28: 'reserved ',
                29: 'reserved ',
                30: 'reserved ',
                31: 'cloud humidity profiles (1000 hPa)',
                32: 'cloud humidity profiles (850 hPa)',
                33: 'cloud humidity profiles (700 hPa)',
                34: 'cloud humidity profiles (600 hPa)',
                35: 'cloud humidity profiles (500 hPa)',
                36: 'cloud humidity profiles (400 hPa)',
                37: 'cloud humidity profiles (300 hPa)',
                101: 'clear sky environment monitoring datasets',
                201: 'ATOVS (1000-10 hPa) temperature fields (K)',
                202: 'ATOVS (1000-10 hPa) temperature fields (K)',
                203: 'ATOVS (1000-10 hPa) temperature fields (K)',
                204: 'ATOVS (1000-10 hPa) temperature fields (K)',
                205: 'ATOVS (1000-10 hPa) temperature fields (K)',
                206: 'ATOVS (1000-10 hPa) temperature fields (K)',
                207: 'ATOVS (1000-10 hPa) temperature fields (K)',
                208: 'ATOVS (1000-10 hPa) temperature fields (K)',
                209: 'ATOVS (1000-10 hPa) temperature fields (K)',
                210: 'ATOVS (1000-10 hPa) temperature fields (K)',
                211: 'ATOVS (1000-10 hPa) temperature fields (K)',
                212: 'ATOVS (1000-10 hPa) temperature fields (K)',
                213: 'ATOVS (1000-10 hPa) temperature fields (K)',
                214: 'ATOVS (1000-10 hPa) temperature fields (K)',
                215: 'ATOVS (1000-10 hPa) temperature fields (K)',

                401: 'ATOVS (1000 ~ 300 hPa) dew point temperature fields (K)',
                402: 'ATOVS (1000 ~ 300 hPa) dew point temperature fields (K)',
                403: 'ATOVS (1000 ~ 300 hPa) dew point temperature fields (K)',
                404: 'ATOVS (1000 ~ 300 hPa) dew point temperature fields (K)',
                405: 'ATOVS (1000 ~ 300 hPa) dew point temperature fields (K)',
                406: 'ATOVS (1000 ~ 300 hPa) dew point temperature fields (K)',

                501: 'ATOVS atmospheric stability index (dimensionless)',
                502: 'ATOVS clear sky total atmospheric column water vapor content (mm)',
                503: 'ATOVS total atmospheric column ozone content (Db)',
                504: ' ATOVS outgoing longwave radiation (W/m2)',
                505: 'ATOVS cloud top height (hPa) ',
                506: 'ATOVS cloud top temperature (K)',
                507: 'ATOVS cloudiness (dimensionless)(ZK)',
                }



# -*- coding: utf-8 -*-
# @Author: wqshen
# @Email: wqshen91@gmail.com
# @Date: 2021/5/31 15:47
# @Last Modified by: wqshen

import os
import pytest
from awx import Awx


class TestReader:

    def test_read_product_3_fy2g_tbb(self):
        fpath = r'./data/FY2G_TBB_IR1_OTG_20150729_0000.AWX'
        ds = Awx(pathfile=fpath)
        print(ds)
        print(ds.values)
        print(ds.sel(lat=slice(20, 40), lon=slice(100, 130)))

    def test_read_product_3_fy2e_cta(self):
        fpath = r'./data/FY2E_CTA_MLT_OTG_20170126_0130.AWX'
        ds = Awx(pathfile=fpath)
        print(ds)
        print(ds.values)
        print(ds.sel(lat=slice(20, 40), lon=slice(100, 130)))

    def test_read_product_1_fy2_ir(self):
        fpath = r'./data/ANI_IR2_R01_20230217_0800_FY2G.AWX'
        ds = Awx(pathfile=fpath)
        print(ds)
        print(ds.values)
        print(ds.sel(lat=slice(20, 40), lon=slice(100, 130)))

    def test_read_product_1_fy2_vis(self):
        fpath = r'./data/ANI_VIS_R02_20230217_1000_FY2G.AWX'
        ds = Awx(pathfile=fpath)
        print(ds)
        print(ds.values)
        print(ds.sel(lat=slice(20, 40), lon=slice(100, 130)))

    def test_plot_matplotlib(self):
        import matplotlib.pyplot as plt

        fpath = r'./data/ANI_VIS_R02_20230217_1000_FY2G.AWX'
        ds = Awx(pathfile=fpath)
        print(ds)
        dar = ds.values.squeeze()
        plt.pcolormesh(dar.lon, dar.lat, dar, cmap='Greys_r')
        plt.savefig('ANI_VIS_R02_20230217_1000_FY2G_NoProj.png', dpi=300)
        plt.show()

    def test_plot_cartopy(self):
        import cartopy.crs as ccrs
        import matplotlib.pyplot as plt

        # fpath = r'./data/ANI_VIS_R02_20230217_1000_FY2G.AWX'  # Mercator
        fpath = r'./data/ANI_IR2_R01_20230217_0800_FY2G.AWX'  # lambert
        ds = Awx(pathfile=fpath)
        print(ds)
        dar = ds.values.squeeze()

        plt.figure(figsize=(8, 8))

        if dar.projection == 1:
            proj = ccrs.LambertConformal(central_longitude=dar.clon / 100,
                                         central_latitude=dar.clat / 100,
                                         standard_parallels=(dar.std_lat1_or_lon / 100.,
                                                             dar.std_lat2 / 100.))
            extent = [dar.x.min(), dar.x.max(), dar.y.min(), dar.y.max()]
        elif dar.projection == 2:
            proj = ccrs.Mercator(central_longitude=dar.clon / 100,
                                 latitude_true_scale=dar.std_lat1_or_lon / 100.)
            extent = [dar.x.min(), dar.x.max(), dar.y.min(), dar.y.max()]
        elif dar.projection == 4:
            proj = ccrs.PlateCarree(central_longitude=dar.clon / 100.)
            extent = [dar.lon.min(), dar.lon.max(), dar.lat.min(), dar.lat.max()]
        else:
            raise NotImplementedError()
        ax = plt.axes(projection=proj)
        ax.set_extent(extent, crs=proj)
        ax.coastlines(resolution='110m')
        ax.gridlines(draw_labels=True)
        ax.pcolormesh(dar.x, dar.y, dar, cmap='Greys_r')
        plt.savefig(os.path.splitext(os.path.basename(fpath))[0] + '.png', dpi=300, bbox_inches='tight')
        plt.show()


if __name__ == '__main__':
    pytest.main(['-q', 'test_diamond_reader.py'])



# -*- coding: utf-8 -*-
# @Author: wqshen
# @Email: wqshen91@gmail.com
# @Date: 2023/2/14 16:22
# @Last Modified by: wqshen


import os
import sys
import argparse
import numpy as np
import xarray as xr
from os import PathLike
from datetime import datetime
from struct import calcsize, unpack
from pyproj import CRS, Transformer
from typing import cast, Optional, Union
from dataclasses import dataclass, fields
from .awx_geos_image import AwxGeosImageHead
from .awx_polar_image import AwxPolarImageHead
from .awx_grid_field import AwxGridHead
from .awx_discrete_field import AwxDiscreteHead


@dataclass
class AwxFileName:
    satellite: str
    product_type: str
    channel: str
    proj: str
    obs_date: str
    obs_start_time: str

    def __post_init__(self):
        self._cast_fields_types()

    def _cast_fields_types(self):
        for field in fields(cast(dataclass, self)):
            field_value = getattr(self, field.name)
            setattr(self, field.name, field.type(field_value))


@dataclass
class AwxHead:
    filename_Sat96: str
    int_order: int = 0
    head1_length: int = 40
    head2_length: int = 0
    filled_length: int = 0
    record_length: int = 0
    head_record: int = 40
    data_record: int = 0
    product_kind: int = 0
    zip_method: int = 0
    form_string: str = "SAT2004"
    data_quality: int = 0

    @staticmethod
    def dtype() -> str:
        return '<12s9h8sh'

    def __post_init__(self):
        self._cast_fields_types()

    def _cast_fields_types(self):
        for field in fields(cast(dataclass, self)):
            field_value = getattr(self, field.name)
            if isinstance(field_value, bytes):
                field_value = field_value.rstrip(b'\x00').decode('utf8')
            setattr(self, field.name, field.type(field_value))


@dataclass
class AwxPositioning:
    coordination: int = 0
    source: int = 0
    resolution: int = 0
    ul_lat: int = 0
    ul_lon: int = 0
    size_h: int = 0
    size_v: int = 0
    reserve: int = 0

    def __post_init__(self):
        self._cast_fields_types()

    def _cast_fields_types(self):
        for field in fields(cast(dataclass, self)):
            field_value = getattr(self, field.name)
            if isinstance(field_value, bytes):
                field_value = field_value.rstrip(b'\x00').decode('utf8')
            setattr(self, field.name, field.type(field_value))


@dataclass
class Awx(object):
    pathfile: Optional[Union[str, PathLike]] = None
    name: Optional[AwxFileName] = None
    head1: AwxHead = None
    head2: Union[AwxGeosImageHead, AwxPolarImageHead, AwxGridHead] = None
    palette: Optional[np.ndarray] = None
    calibration: Optional[np.ndarray] = None
    calibrating: Optional[bool] = False
    positioning: Optional[AwxPositioning] = None
    data: Union[np.ndarray, dict] = None
    missing_value: Optional[float] = None
    _ds: Optional[xr.DataArray] = None

    def __post_init__(self):
        if self.pathfile is not None:
            self.read(self.pathfile)

    def sel(self, lat=None, lon=None, method=None, **kwargs):
        """interface to select variable from file by given more filter and clip parameters

        Parameters
        ----------
        lat (slice, optional): latitude extent
        lon (slice, optional): longitude extent
        method (str): method in processing un-precise boundary, see `xarray.DataArray.sel`
        kwargs (dict): other parameters used to process netcdf variable

        Returns
        -------
        ncvar (xarray.DataArray): Readed variable in xarray.DataArray
        """
        darray = self.clip(self._ds, lat, lon, method)
        ncvar = darray.sel(**kwargs)

        return ncvar

    def clip(self, darray, lat=None, lon=None, method=None):
        """Clip the longitude and latiutde of AWX DataArray

        Parameters
        ----------
        darray (xarray.DataArray): DataArray with longitude and latitude coordinates
        lat (slice, optional): latitude extent
        lon (slice, optional): longitude extent
        method (str): method in processing un-precise boundary, see `xarray.DataArray.sel`

        Returns
        -------
        darray (xarray.DataArray): regional clipped DataArray
        """
        if lat is not None or lon is not None:
            if len(darray['lat'].shape) == 2:
                if isinstance(lat, slice):
                    start, stop = min(lat.start, lat.stop), max(lat.start, lat.stop)
                    darray = darray.where((darray['lat'] >= start) & (darray['lat'] <= stop),
                                          drop=True)
                if isinstance(lon, slice):
                    darray = darray.where(
                        (darray['lon'] >= lon.start) & (darray['lon'] <= lon.stop),
                        drop=True)
            else:
                if lat is not None:
                    if darray.indexes['lat'].is_monotonic_decreasing:
                        lat = slice(lat.stop, lat.start, lat.step)
                    darray = darray.sel({'lat': lat}, method=method)
                if lon is not None:
                    darray = darray.sel({'lon': lon}, method=method)
        return darray

    def read(self, path_or_bytes: Union[str, PathLike, bytes]):
        """read AWX file or bytes

        Parameters
        ----------
        path_or_bytes: path to AWX file or data in bytes
        """
        if isinstance(path_or_bytes, (str, PathLike)):
            self._deconstruct_filename(path_or_bytes)
            with open(self.pathfile, 'rb') as f:
                bytes_array = f.read()
        else:
            bytes_array = path_or_bytes

        head1_type = AwxHead.dtype()
        p = calcsize(head1_type)
        head1_data = unpack(head1_type, bytes_array[:p])
        self.head1 = AwxHead(*head1_data)

        Head2 = self.head2_interface()
        if Head2 is None:
            raise NotImplementedError("Unsupported file format.")
        head2_type = Head2.dtype()
        p_n = p + calcsize(head2_type)
        head2_data = unpack(head2_type, bytes_array[p:p_n])
        p = p_n
        self.head2 = Head2(*head2_data)

        pp = self.head1.head_record * self.head1.record_length
        if self.head1.product_kind in [1, 2]:
            if self.head2.palette_data_length > 0:
                p_n = p + self.head2.palette_data_length
                self.palette = np.frombuffer(bytes_array[p:p_n],
                                             dtype=np.uint8, count=self.head2.palette_data_length)
                p = p_n
            if self.head2.calibration_data_length > 0:
                p_n = p + self.head2.calibration_data_length
                self.calibration = np.frombuffer(bytes_array[p:p_n], dtype=np.uint16,
                                                 count=int(self.head2.calibration_data_length / 2))
                p = p_n
            if self.head2.position_data_length > 0:
                p_n = p + 16
                self.positioning = AwxPositioning(*unpack('8h', bytes_array[p:p_n]))
            n_obs = self.head2.width * self.head2.height
            data = np.frombuffer(bytes_array[pp: pp + n_obs], dtype=np.uint8, count=n_obs)
            if self.calibrating:
                data = self.calibration[data]
            data = np.array(data).reshape(self.head2.height, self.head2.width)
        elif self.head1.product_kind == 3:
            n_obs = self.head2.size_h * self.head2.size_v
            body_data = np.frombuffer(bytes_array[pp: pp + n_obs * self.head2.grid_byte],
                                      dtype=f'u{self.head2.grid_byte}', count=n_obs).astype('f')
            if self.head2.has_quality:
                down, up = self.head2.quality_down, self.head2.quality_up
                body_data[(body_data < down) | (body_data > up)] = np.nan
            data = (body_data + self.head2.grid_base) / self.head2.grid_scale
            data = np.array(data).reshape(self.head2.size_v, self.head2.size_h)
        else:
            raise NotImplementedError("Not supported product kind.")
        self.data = data
        self._ds = self.to_xarray()
    def to_xarray(self) -> xr.DataArray:
        """convert to xarray.DataArray"""
        attrs = self.head1.__dict__
        attrs.update(self.head2.__dict__)
        if self.head1.product_kind in (1, 2):
            coords = self.image_coordinate()
            if 'x' in coords and 'y' in coords:
                attrs['proj4'] = coords.pop('proj4')
                return xr.DataArray(self.data[None, :, :], dims=('time', 'y', 'x'),
                                    coords=coords, attrs=attrs)
            else:
                return xr.DataArray(self.data[None, :, :],
                                    dims=('time', 'lat', 'lon'),
                                    coords=coords, attrs=attrs)
        elif self.head1.product_kind == 3:
            attrs['description'] = self.head2.grid_elements_dict()[self.head2.grid_element]
            return xr.DataArray(self.data[None, :, :], dims=('time', 'lat', 'lon'),
                                coords=self.grid_coordinate(), attrs=attrs)
        else:
            raise NotImplementedError("Not supported product kind.")

    def grid_coordinate(self) -> dict:
        """calculate AWX grid field coordination"""
        h2 = self.head2
        time = datetime(h2.start_year, h2.start_month, h2.start_day, h2.start_hour, h2.start_minute)
        lats = np.linspace(h2.ul_lat / 100., h2.lr_lat / 100., h2.size_v)
        lons = np.linspace(h2.ul_lon / 100., h2.lr_lon / 100., h2.size_h)
        return {'time': [time], 'lat': lats, 'lon': lons}

    def image_coordinate(self) -> dict:
        """calculate AWX image field coordination"""
        h2 = self.head2
        if hasattr(h2, 'year'):
            time = datetime(h2.year, h2.month, h2.day, h2.hour, h2.minute)
        else:
            time = datetime(h2.start_year, h2.start_month, h2.start_day,
                            h2.start_hour, h2.start_minute)
        proj_str = self.proj_str()
        if proj_str is None:
            lats = np.linspace(h2.ul_lat / 100., h2.lr_lat / 100., h2.height)
            lons = np.linspace(h2.ul_lon / 100., h2.lr_lon / 100., h2.width)
            return {'time': [time], 'lat': lats, 'lon': lons}
        else:
            proj = CRS.from_proj4(proj_str)
            transformer = Transformer.from_crs("EPSG:4326", proj, always_xy=True)
            cx, cy = transformer.transform(h2.clon / 100., h2.clat / 100.)
            dx, dy = h2.reso_h / 100. * 1000, h2.reso_v / 100. * 1000,
            ll_x = cx - (dx * h2.width / 2.)
            ll_y = cy - (dy * h2.height / 2.)
            ur_x = cx + (dx * h2.width / 2.)
            ur_y = cy + (dy * h2.height / 2.)
            x = np.linspace(ll_x, ur_x, h2.width)
            y = np.linspace(ur_y, ll_y, h2.height)

            transformer = Transformer.from_crs(proj, "EPSG:4326", always_xy=True)
            x2d, y2d = np.meshgrid(x, y)
            lons, lats = transformer.transform(x2d, y2d)
            lons = xr.DataArray(lons, dims=('y', 'x'), coords={'y': y, 'x': x})
            lats = xr.DataArray(lats, dims=('y', 'x'), coords={'y': y, 'x': x})
            return {'time': [time], 'lat': lats, 'lon': lons, 'x': x, 'y': y, 'proj4': proj_str}

    def proj_scale_factor(self, std_parallel):
        """calculate projection scale factor"""
        from math import pi, sin, cos, tan, pow

        e = 0.081819191
        std_parallel = std_parallel * pi / 180.
        if std_parallel > 0:
            tf = tan(pi / 4. - std_parallel / 2.) * (
                pow((1. + e * sin(std_parallel)) / (1. - e * sin(std_parallel)), e / 2.))
        else:
            tf = tan(pi / 4. + std_parallel / 2.) * (
                pow((1. + e * sin(std_parallel)) / (1. - e * sin(std_parallel)), e / 2.))
        mf = cos(std_parallel) / pow(1. - e * e * pow(sin(std_parallel), 2.), 0.5)
        k0 = mf * (pow(pow(1. + e, 1. + e) * pow(1. - e, 1. - e), 0.5)) / (2. * tf)
        return k0

    def proj_str(self):
        """generate projection string to construct Proj"""
        h2 = self.head2
        proj_type = h2.projection
        if proj_type == 1:
            proj = f'+proj=lcc +lon_0={h2.clon / 100.} +lat_0={h2.clat / 100.} ' \
                   f'+lat_1={h2.std_lat1_or_lon / 100.} +lat_2={h2.std_lat2 / 100.}'
        elif proj_type == 2:
            proj = f'+proj=merc +lon_0={h2.clon / 100.} +lat_ts={h2.std_lat1_or_lon / 100.}'
        elif proj_type == 3:
            proj = f'+proj=stere +lat_0={h2.clat / 100.} +lon_0={h2.clon / 100.} ' \
                   f'+k={self.proj_scale_factor(h2.std_lat1_or_lon / 100.)}'
        elif proj_type == 4:
            proj = None
        else:
            raise NotImplementedError("Not supported projection type.")
        return proj

    def head2_interface(self):
        kind = self.head1.product_kind
        heads = [None, AwxGeosImageHead, AwxPolarImageHead, AwxGridHead, None, None]
        return heads[kind]

    def _deconstruct_filename(self, filename):
        """deconstruct AWX filename to get base information of file"""
        self.name = AwxFileName(*os.path.splitext(os.path.basename(filename))[0].split('_'))

    @property
    def values(self) -> xr.DataArray:
        return self._ds

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Close open dataset"""
        del self.data, self.head1, self.head2, self.palette, self.calibration
        del self.positioning, self._ds, self.pathfile

    def __repr__(self) -> str:
        """print"""
        return self.head1.__repr__() + '\n' + self.head2.__repr__()


def _info():
    example_text = """Example:
     awx_info FY2G_TBB_IR1_OTG_20150729_0000.AWX
     """
    parser = argparse.ArgumentParser(description='Model Data Dumper', epilog=example_text,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('filename', help='awx file name')
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    args = parser.parse_args()
    awx = Awx(args.filename)
    print(awx.head1, '\n', awx.head2, '\n', awx._ds)


def _convert_to_nc():
    example_text = """Example:
     awx_to_nc FY2G_TBB_IR1_OTG_20150729_0000.AWX FY2G_TBB_IR1_OTG_20150729_0000.nc
     """
    parser = argparse.ArgumentParser(description='Model Data Dumper', epilog=example_text,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('filename', help='awx file name')
    parser.add_argument('outfile', help='path to convert netcdf file name')
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    args = parser.parse_args()
    awx = Awx(args.filename)
    ds = awx.values
    ds.name = 'AWX'
    ds = ds.to_dataset()
    comp = dict(zlib=True, complevel=5)
    encoding = {var: comp for var in ds.data_vars}
    ds.to_netcdf(args.outfile, encoding=encoding)



# -*- coding: utf-8 -*-
# @Author: wqshen
# @Date: 2019/4/6 11:59
# @Last Modified by:   wqshen


from os.path import exists
from setuptools import setup, find_packages

name = 'awx'
version = "0.1"

install_requires = [
    'numpy',
    'xarray',
    'pyproj'
]

classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Science/Research/Operation',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Topic :: GeoScience :: AWX Data Reader',
]

setup(
    name=name,
    version=version,
    description='AWX Satellite Data Reader',
    author='wqshen',
    author_email='wqshen91@163.com',
    long_description=open('README.rst').read() if exists('README.rst') else '',
    url='https://github.com/wqshen/AwxReader',
    python_requires='>=3.7',
    install_requires=install_requires,
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'awx_info=awx:_info',
            'awx_to_nc=awx:_convert_to_nc',
        ]
    },
    include_package_data=True,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# @Author: wqshen
# @Email: wqshen91@gmail.com
# @Date: 2023/2/9 22:07
# @Last Modified by: wqshen

# Note: Not finished.


from typing import cast
from dataclasses import dataclass, fields


@dataclass
class AwxDiscreteHead:
    satellite_name: str
    element: int
    chars_by_record: int
    points: int
    start_year: int
    start_month: int
    start_day: int
    start_hour: int
    start_minute: int
    end_year: int
    end_month: int
    end_day: int
    end_hour: int
    end_minute: int
    inversion_method: int
    initial_field: int
    default: int = 0

    @staticmethod
    def dtype() -> str:
        return '=8s16h'

    def __post_init__(self):
        self._cast_fields_types()

    def _cast_fields_types(self):
        for field in fields(cast(dataclass, self)):
            field_value = getattr(self, field.name)
            if isinstance(field_value, bytes):
                field_value = field_value.rstrip(b'\x00').decode('utf8')
            setattr(self, field.name, field.type(field_value))


@dataclass
class AwxAtovs:
    pass


@dataclass
class AwxWind:
    pass



# -*- coding: utf-8 -*-
# @Author: wqshen
# @Email: wqshen91@gmail.com
# @Date: 2023/2/9 22:07
# @Last Modified by: wqshen

from typing import cast
from dataclasses import dataclass, fields


@dataclass
class AwxGeosImageHead:
    satellite_name: str
    year: int
    month: int
    day: int
    hour: int
    minute: int
    channel: int
    projection: int
    width: int
    height: int
    ul_row_id: int
    ul_pixel_id: int
    sampling_rate: int
    ul_lat: int
    lr_lat: int
    ul_lon: int
    lr_lon: int
    clat: int
    clon: int
    std_lat1_or_lon: int
    std_lat2: int
    reso_h: int
    reso_v: int
    mark_geogrid_overlap: int = 0
    value_geogrid_overlap: int = 0
    palette_data_length: int = 0
    calibration_data_length: int = 0
    position_data_length: int = 0
    reserved: int = 0

    @staticmethod
    def dtype() -> str:
        return '=8s28h'

    def __post_init__(self):
        self._cast_fields_types()

    def _cast_fields_types(self):
        for field in fields(cast(dataclass, self)):
            field_value = getattr(self, field.name)
            if isinstance(field_value, bytes):
                field_value = field_value.rstrip(b'\x00').decode('utf8')
            setattr(self, field.name, field.type(field_value))
